package edu.iastate.jrelm.rl;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.core.State;
import edu.iastate.jrelm.core.StateDomain;

/**
 * The StatelessPolicy is a specialization of the general Policy that doesn't make use 
 * of States. This means, as an interface, StatelessPolicy inherits extra State related
 * methods from Policy that will never be used. This abstract class is provided as a 
 * convenience to implement away the extra methods for those who want to implement a 
 * StatelessPolicy. 
 *  
 * @author Charles Gieseler
 *
 */
public abstract class AbstractStatelessPolicy
		<I, A extends Action> 
		implements StatelessPolicy<I,A> {

	/* ** State related methods ** */
	public A generateAction(Object stateID) {
		return generateAction();
	}
	
	public StateDomain<Object, State> getStateDomain() {return null;}

	public double getProbability(Object stateID, I actionID) {
		return getProbability(actionID);
	}

	public void setProbability(Object stateID, I actionID, double newValue) {
		setProbability(actionID, newValue);
	}
}
