package edu.iastate.jrelm.rl.rotherev.advanced;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.spillover.SpilloverWeightGenerator;

/**
 * 
 * @author Charles Gieseler
 *
 * @param <A> - the type of Action to compare
 */
public class StandardSpillover<A extends Action> extends SpilloverWeightGenerator<A> {

	double e;
	double dSize;
	
	/**
	 * 
	 * @param experimentation - experimentation value.
	 * @param domainSize - the number of possible Actions to choose from
	 */
	public StandardSpillover(double experimentation, int domainSize){
		e = experimentation;
		dSize = domainSize;
	}
	
	public double generateWeight(A action) {
		double weight  = 0;
		A refAction = getReferenceAction();
		
		if(refAction.equals(action) || (dSize==1)){
			weight = (1-e);
		}else{
			weight = (e /(dSize - 1));
		}
		return weight;
	}
	
	/* **********************************************
	 * 				ACCESSOR METHODS
	 * **********************************************/

	/**
	 * Get the current value for the domain size.
	 */
	public double getDominSize() {
		return dSize;
	}
	
	/**
	 * Set the current value for the domain size. 
	 * Note: If dynamic domains are used, this spillover weight generator 
	 * will need to be updated whenever the domain size changes. 
	 * 
	 * @param size - size of the domain
	 */
	public void setDomainSize(double size) {
		dSize = size;
	}
	
	/**
	 * Get the current value for the experimentation parameter.
	 * 
	 * @return the experimentation value
	 */
	public double getExperimentation() {
		return e;
	}
	
	/**
	 * Set the current value for the experimentation parameter. 
	 * Note: Experimentation may change over the course of a simulation.
	 * This spill over weight generator will need to be updated anytime 
	 * the experimentation parameter changes. 
	 * 
	 * @param exp - new value for the experimentation parameter.
	 */
	public void setExperimentation(double exp) {
		this.e = exp;
	}
	
	public String getName(){return "Roth-Erev";}
	
}
